@@ -55,16 +55,27 @@ def encode_connection_string(fields):
         '%s=%s' % (k, encode_value(v))
         for k, v in fields.items()
     )
+def prepare_token_for_odbc(token):
+    """
+    Will prepare token for passing it to the odbc driver, as it expects
+    bytes and not a string
+    :param token:
+    :return: packed binary byte representation of token string
+    """
+    if not isinstance(token, str):
+        raise TypeError("Invalid token format provided.")
 
+    tokenstr = token.encode()
+    exptoken = b""
+    for i in tokenstr:
+        exptoken += bytes({i})
+        exptoken += bytes(1)
+    return struct.pack("=i", len(exptoken)) + exptoken
 
 def encode_value(v):
     """If the value contains a semicolon, or starts with a left curly brace,
     then enclose it in curly braces and escape all right curly braces.
     """
-    print('v--> ', v)
-    if type(v) is dict:
-        print('Skipped for access token by it dict')
-        return v
     if ';' in v or v.strip(' ').startswith('{'):
         return '{%s}' % (v.replace('}', '}}'),)
     return v
@@ -276,6 +287,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         options = conn_params.get('OPTIONS', {})
         driver = options.get('driver', 'ODBC Driver 17 for SQL Server')
         dsn = options.get('dsn', None)
+        options_extra_params = options.get('extra_params', '')
 
         # Microsoft driver names assumed here are:
         # * SQL Server Native Client 10.0/11.0
@@ -314,13 +326,12 @@ class DatabaseWrapper(BaseDatabaseWrapper):
                 exp_token += bytes({i})
                 exp_token += bytes(1)
             token_struct = struct.pack("=i", len(exp_token)) + exp_token
-            cstr_parts['attrs_before'] = {1256: bytearray(token_struct)}
         elif user:
             cstr_parts['UID'] = user
-            if 'Authentication=ActiveDirectoryInteractive' not in options.get('extra_params', ''):
+            if 'Authentication=ActiveDirectoryInteractive' not in options_extra_params:
                 cstr_parts['PWD'] = password
-        else:
-            if ms_drivers.match(driver):
+        elif 'TOKEN' not in conn_params:
+            if ms_drivers.match(driver) and 'Authentication=ActiveDirectoryMsi' not in options_extra_params:
                 cstr_parts['Trusted_Connection'] = trusted_connection
             else:
                 cstr_parts['Integrated Security'] = 'SSPI'
@@ -343,15 +354,31 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         retries = options.get('connection_retries', 5)
         backoff_time = options.get('connection_retry_backoff_time', 5)
         query_timeout = options.get('query_timeout', 0)
+        setencoding = options.get('setencoding', None)
+        setdecoding = options.get('setdecoding', None)
 
         conn = None
         retry_count = 0
         need_to_retry = False
+        args = {
+            'unicode_results': unicode_results,
+            'timeout': timeout,
+        }
+        if 'TOKEN' in conn_params:
+            args['attrs_before'] = {
+                1256: prepare_token_for_odbc(conn_params['TOKEN'])
+            }
         while conn is None:
             try:
-                conn = Database.connect(connstr,
-                                        unicode_results=unicode_results,
-                                        timeout=timeout)
+                if is_azure_based_token is not None and is_azure_based_token is True:
+                    conn = Database.connect(connstr,
+                                            attrs_before={1256: bytearray(token_struct)},
+                                            unicode_results=unicode_results,
+                                            timeout=timeout)
+                else:
+                    conn = Database.connect(connstr,
+                                            unicode_results=unicode_results,
+                                            timeout=timeout)
             except Exception as e:
                 for error_number in self._transient_error_numbers:
                     if error_number in e.args[1]:
@@ -366,6 +393,12 @@ class DatabaseWrapper(BaseDatabaseWrapper):
                     raise
 
         conn.timeout = query_timeout
+        if setencoding:
+            for entry in setencoding:
+                conn.setencoding(**entry)
+        if setdecoding:
+            for entry in setdecoding:
+                conn.setdecoding(**entry)
         return conn
 
     def init_connection_state(self):
